@@ -5,7 +5,7 @@ package threads::shared;  # but we're masquerading as threads::shared.pm
 # Compatibility with the standard threads::shared
 # Do everything by the book from now on
 
-$VERSION  = '0.15';
+$VERSION  = '0.16';
 $threads_shared = $threads_shared = 1;
 use strict;
 
@@ -359,7 +359,7 @@ sub _remote {
 #---------------------------------------------------------------------------
 #  IN: 1 message to display
 
-sub _croak { goto &Carp::confess } #_croak
+sub _croak { return &Carp::confess } #_croak
 
 #---------------------------------------------------------------------------
 
